package Data::Filter::Formater;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Filter::Formater - Format filter tree into HTML hierarchical structure

=head1 SYNOPSIS

 use Data::Filter::Formater;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Data::Filter::Traverser(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Data::Filter::Traverser;
use Data::Filter::Factory;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Data::Filter::Traverser');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item format_html($$) [PUBLIC]

  Usage     : my $result = $instance->format_html($filter)
  Purpose   : Format filter tree into HTML hierarchical structure
  Arguments : Data::Filter $filter - query filter tree
  Returns   : STRING html
  Throws    : Croaks, if not invoked on object

=cut

sub format_html($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter,) = @_;

    return '<div class="filter-tree">'.$filter->traverse($self).'</div>';
}

#-------------------------------------------------------------------------------

=item FILTER CALLBACKS [PUBLIC,ABSTRACT]

  Usage     : return $traverser->$name(@args);
  Purpose   : Rule tree traversal callbacks, for each rule class one callback method
  Arguments : Depend on particular callback
  Returns   : Any result from callback will be passed to the parent node callback as argument
  Throws    : Croaks, if invoked on object
  Comments  : These methods are called from Data::Filter::traverse method
              during filter tree traversal. Arguments are dependent on

=cut

sub constant($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter,) = @_;

    return '<div class="filter-constant">'.$filter->value_raw().'</div>';
}
sub variable($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter,) = @_;

    return '<div class="filter-variable">'.$filter->value().'</div>';
}
sub list($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter,) = @_;

    my $str = '<ul class="filter-list">';
    foreach my $i (@{$filter->value_raw()}) {
        $str .= "<li>$i</li>"
    }
    $str .= '</ul>';
    return $str;
}
sub existence($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter,) = @_;

    return '<div class="filter-existence">'.$filter->value().'</div>';
}
# ---
sub operationUnary($$$$;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, $operation, $subfilter, $result) = @_;

    return '<div class="filter-operation"><h3>'.$operation.'</h3><ul class="optree"><li>'.$result.'</li></ul></div>';
}
sub operationBinary($$$$$$;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($filter, $operation, $l_subfilter, $r_subfilter, $l_result, $r_result) = @_;

    return '<div class="filter-operation"><h3>'.$operation.'</h3>'.
           '<ul class="optree"><li>'.$l_result.'</li><li>'.$r_result.'</li></ul></div>';
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Data::Filter::Formater instance
#  Arguments :
#  Returns   : Data::Filter::Formater reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self;
}

=pod

=back

=cut

1;
