package Data::Filter::Rule::OperationBinary;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Filter::Rule::OperationBinary - Mentat message filter tree binary operation node

=head1 SYNOPSIS

 use Data::Filter::Rule::OperationBinary;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1), Data::Filter::Rule(3), Data::Filter::Rule::Operation(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #--- DEVELOPMENT ONLY ---#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Data::Filter::Rule::Operation;
use Data::Core qw(:c_dtypes :c_opers);

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# List of valid operation types and their normalized names
my %OPERATIONS = (
        'and'   => OP_AND,
        'AND'   => OP_AND,
        '&&'    => OP_AND,
        'or'    => OP_OR,
        'OR'    => OP_OR,
        '||'    => OP_OR,
        'xor'   => OP_XOR,
        'XOR'   => OP_XOR,
        '^^'    => OP_XOR,

        'eq'    => OP_EQ,
        'EQ'    => OP_EQ,
        '=='    => OP_EQ,
        'ne'    => OP_NE,
        'NE'    => OP_NE,
        '!='    => OP_NE,
        'gt'    => OP_GT,
        'GT'    => OP_GT,
        '>'     => OP_GT,
        'lt'    => OP_LT,
        'LT'    => OP_LT,
        '<'     => OP_LT,
        'ge'    => OP_GE,
        'GE'    => OP_GE,
        '>='    => OP_GE,
        'le'    => OP_LE,
        'LE'    => OP_LE,
        '<='    => OP_LE,

        'like'  => OP_LIKE,
        'LIKE'  => OP_LIKE,
        '=~'    => OP_LIKE,

        'in'    => OP_IN,
        'IN'    => OP_IN,
        '~~'    => OP_IN,

        '+'     => OP_ADD,
        '-'     => OP_SUB,
        '*'     => OP_MUL,
        '/'     => OP_DIV,
        '%'     => OP_MOD,
        '.'     => OP_CON,
    );

# List of appropriate result datatypes for given operations
my %DATATYPES = (
        OP_AND, DT_BOOL,
        OP_OR,  DT_BOOL,
        OP_XOR, DT_BOOL,

        OP_EQ, DT_BOOL,
        OP_NE, DT_BOOL,
        OP_GT, DT_BOOL,
        OP_LT, DT_BOOL,
        OP_GE, DT_BOOL,
        OP_LE, DT_BOOL,

        OP_LIKE, DT_BOOL,

        OP_IN, DT_BOOL,

        OP_ADD, DT_NUMERIC,
        OP_SUB, DT_NUMERIC,
        OP_MUL, DT_NUMERIC,
        OP_DIV, DT_NUMERIC,
        OP_MOD, DT_NUMERIC,
        OP_CON, DT_STRING,
    );

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Data::Filter::Rule::Operation');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item valid_operation($) [PUBLIC]

  Usage     : my $operation = $rule->valid_operation();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub valid_operation($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($operation,) = @_;
    confess "Invalid binary operation '$operation'" unless $OPERATIONS{$operation};

    return ($OPERATIONS{$operation}, $DATATYPES{$OPERATIONS{$operation}});
}

#-------------------------------------------------------------------------------

=item left_operand($) [PUBLIC]

  Usage     :
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub left_operand(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($operand,) = @_;

    # Setter mode
    if ($operand) {
        confess "Left operand must be Data::Filter::Rule" unless $operand->isa('Data::Filter::Rule');
        $self->{LEFT_OPERAND} = $operand;
    }
    return $self->{LEFT_OPERAND};
}

=item right_operand($) [PUBLIC]

  Usage     :
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub right_operand(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($operand,) = @_;

    # Setter mode
    if ($operand) {
        confess "Right operand must be Data::Filter::Rule" unless $operand->isa('Data::Filter::Rule');
        $self->{RIGHT_OPERAND} = $operand;
    }
    return $self->{RIGHT_OPERAND};
}

#-------------------------------------------------------------------------------

=item traverse($;@) [PUBLIC]

  Usage     : my $result = Data::Filter::Rule->traverse($traverser);
  Purpose   : Traversal through the filter rule tree with given traverser object
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub traverse($;@)
{
    my $self    = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($traverser, @args) = @_;
    confess "Traverser object must be given as first argument" unless ref $traverser;

    my $lo = $self->left_operand();
    my $ro = $self->right_operand();

    my $name = lcfirst($self->name());
    confess "Traverser object can`t handle '$name' callback hook" unless $traverser->can($name);

    my $lr = $lo->traverse($traverser, @args);
    my $rr = $ro->traverse($traverser, @args);

    return $traverser->$name($self, $self->operation(), $lo, $ro, $lr, $rr, @args);
}

#-------------------------------------------------------------------------------

=item to_string() [PUBLIC]

  Usage     : my $result = Data::Filter::Rule->to_string();
  Purpose   :
  Returns   :
  Arguments :
  Throws    :
  Comments  :
  See Also  :

=cut

sub to_string()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return "(".$self->left_operand()." ".$self->operation()." ".$self->right_operand().")";
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _init()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($type, $left, $right) = @_;
    confess "Binary operation must have two operands" unless ref $left and ref $right;

    $self->operation($type);
    $self->left_operand($left);
    $self->right_operand($right);

    return $self;
}

=pod

=back

=cut

1;
