package Mentat::Mailer::Report;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Mailer::Report - Short module description

=head1 SYNOPSIS

 use Mentat::Mailer::Report;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Digest::MD5 qw(md5_hex);
use Mail::Sendmail;
use Encode;
use IO::Compress::Zip qw(zip $ZipError);
use MIME::Base64 qw(encode_base64);
use Cwd 'abs_path';

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:func);

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

use constant TYPE_SUMMARY => 'summary';
use constant TYPE_EXTRA   => 'extra';

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#
my $SERVICE_INSTANCE;

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item service() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Message::Formater::Helper->service()
  Purpose   : Create/return reference to the singleton instance
  Arguments : Same as the the new() method
  Returns   : Same as the the new() method
  Throws    : Same as the the new() method
  Comments  : Internally uses new() method
  See Also  : new() method

=cut

sub service()
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my ($config_file, $config_schema_file, $config_over) = @_;
    unless ($SERVICE_INSTANCE) {
        $config_file = loc_cre().'/svc.mentat-mailer-report.json' unless $config_file;
        my $cfg = $class->_load_config($config_file, $config_schema_file, $config_over);
        $SERVICE_INSTANCE = $class->new(%$cfg);
    }
    return $SERVICE_INSTANCE;
}

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Mailer::Report->new();
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Mailer::Report reference on success, croaks on failure
  Throws    : Dies, if not invoked on class
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    ### Creating Template.Module instance...
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Template.Module instance...
}

=item email_report($;%) [PUBLIC]

  Usage     : my ($rc, $result) = $instance->email_report($report, %config);
  Purpose   : Email given report to given abuse contact
  Arguments : Report $report - report [MANDATORY]
              HASH $config - additional configuration [OPTIONAL]
  Returns   : INTEGER $rv containing 1 on SUCCESS and 0 on FAILURE
              STRING $result description
  Throws    : Dies, if not invoked on instance

=cut

sub email_report($;%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report, %config) = @_;
    confess "Report must be given as argument"  unless $report and ref $report eq 'HASH';
    confess "Report ID is missing"              unless $report->{id};
    confess "Report type is missing"            unless $report->{type};
    confess "Report abuse is missing"           unless $report->{abuse};
    confess "Report message is missing"         unless $report->{message};

    my $email = {};

    # Determine actual email destination based on the internal configuration and
    # report metadata
    $self->_setup_destination($report, %config);

    # Setup email headers
    $self->_setup_headers($report, $email, %config);

    # Setup email subject
    $self->_setup_subject($report, $email, %config);

    Log::Loger->debug(__PACKAGE__, "[EML] Report *$report->{id}*: Mail without content: ".Dumper($email));
    #Log::Loger->debug(__PACKAGE__, "[EML] Report *$report->{id}*: ".Dumper($report));

    # Setup email message
    $self->_setup_message($report, $email, %config);

    # Send the email
    return $self->_send_report_email($report, $email, %config);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Mailer::Report instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Mentat::Mailer::Report $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Initializing Template.Module instance...
    my %config = @_;

    die "[EML] Report directory must be specified under configuration key 'reports_dir'\n" unless exists $config{reports_dir};
    die "[EML] Report directory '$config{'reports_dir'}' does not exist\n" unless -d $config{reports_dir};
    die "[EML] Report directory '$config{'reports_dir'}' is not writable\n" unless -w $config{reports_dir};
    die "[EML] Report origin email address must be specified under configuration key 'mail_from'\n" unless $config{mail_from};
    die "[EML] Report reply-to email address must be specified under configuration key 'reply_to'\n" unless $config{reply_to};
    die "[EML] Email address for system administrators must be specified under configuration key 'mail_admin'\n" unless $config{mail_admin};
    die "[EML] Email address for system developers must be specified under configuration key 'mail_devel'\n" unless $config{mail_devel};
    confess "[EML] Email address for test data recipients must be specified under configuration key 'mail_test'" unless $config{mail_test};

    $self->{REPORTS_DIR}    = abs_path($config{reports_dir});
    $self->{FROM}           = $config{mail_from};
    $self->{REPLY_TO}       = $config{reply_to};
    $self->{ADMIN_C}        = $config{mail_admin};
    $self->{DEVEL_C}        = $config{mail_devel};
    $self->{TEST_C}         = $config{mail_test};
    $self->{BCC_C}          = $config{mail_bcc};
    $self->{TEST_DATA}      = $config{test_data};
    $self->{TEST_MODE}      = $config{test_mode};
    $self->{PILOT_MODE}     = $config{pilot_mode};

    if ($self->{REPORTS_DIR} =~ /(.*)/) { $self->{REPORTS_DIR} = $1; }
    Log::Loger->info(__PACKAGE__, "[EML] Using folder '$self->{REPORTS_DIR}' as report attachment source");
    Log::Loger->info(__PACKAGE__, "[EML] Using email '$self->{FROM}' as source email address");
    Log::Loger->info(__PACKAGE__, "[EML] Using email '$self->{REPLY_TO}' as return email address");
    Log::Loger->info(__PACKAGE__, "[EML] Using email '$self->{ADMIN_C}' as admin contact email address");
    Log::Loger->info(__PACKAGE__, "[EML] Using email '$self->{DEVEL_C}' as developer contact email address");
    Log::Loger->info(__PACKAGE__, "[EML] Using email '$self->{TEST_C}' as test data recipient contact email address");
    Log::Loger->info(__PACKAGE__, "[EML] Using email '$self->{BCC_C}' as BCC email address") if $self->{BCC_C};
    Log::Loger->info(__PACKAGE__, "[EML] Running in pilot mode, reports will be send only for certain abuse groups") if $self->{PILOT_MODE};
    Log::Loger->info(__PACKAGE__, "[EML] Running in test data mode, using email '$self->{TEST_C}' as destination email address for all reports") if $self->{TEST_DATA};
    Log::Loger->info(__PACKAGE__, "[EML] Running in test mode, using email '$self->{DEVEL_C}' as destination email address for all reports") if $self->{TEST_MODE};

    return $self;
}

# _load_config($;$) [PROTECTED]
#
#  Usage     : my $config = $self->_load_config($config_file, $config_schema_file);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on instance

sub _load_config($;$)
{
    my $invocant = shift;
    my ($config_file, $config_schema_file, $config_over) = @_;

    my $cfg_parser = Config::JSON->new($config_schema_file);

    # Load the templates from external file
    my ($config, $error);
    #($config, $error) = $cfg_parser->config($config_file, $config_schema_file);
    #die 'Template file syntax error: ' . $error . "\n" unless $config;

    $config = json_load($config_file);

    if ($config_over and ref $config_over eq 'HASH') {
        foreach my $c (keys %$config_over) {
            $config->{$c} = $config_over->{$c};
        }
    }

    return $config;
}

# _setup_destination($;%) [PROTECTED]
#
#  Usage     : my $to = _setup_destination($report);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing

sub _setup_destination($;%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report, %config) = @_;

    # Redirect to Mentat system admin, if abuse address does not exist
    if (not $report->{to} or not $report->{to} =~ /@/) {
        Log::Loger->warn(__PACKAGE__, "[EML] Report *$report->{id}*: Abuse contact address does not exist, redirecting report to system admins at <$self->{ADMIN_C}>");
        $report->{message} = "WARNING: Abuse address did not exist, report was redirected because of privacy reasons to system admins at <$self->{ADMIN_C}>.\n\n$report->{message}";
        $report->{to} = $self->{ADMIN_C};
    }
    # Redirect to Mentat system admin, if abuse address does not exist, but it is
    # because of incompleteness of static whois dump
    elsif ($report->{to} =~ /\?\?\?/) {
        Log::Loger->warn(__PACKAGE__, "[EML] Report *$report->{id}*: Abuse contact address should exist in whois database, but it doesn't, redirecting report to system admins at <$self->{ADMIN_C}>");
        $report->{message} = "WARNING: Abuse address should exist in whois database, but it didn't, report was redirected because of performance reasons to system admins at <$self->{ADMIN_C}>.\n\n$report->{message}";
        $report->{to} = $self->{ADMIN_C};
    }

    # Consider the report redirection, redirect to system admins
    if ($config{redirect}) {
        Log::Loger->warn(__PACKAGE__, "[EML] Report *$report->{id}*: Forcing report redirection from <$report->{to}> to system admins at <$self->{ADMIN_C}>");
        $report->{message} = "WARNING: Report for abuse contact <'$report->{abuse}'> was redirected by configuration from <$report->{to}> to system admins <$self->{ADMIN_C}>.\n\n$report->{message}";
        $report->{to} = $self->{ADMIN_C};
    }
    # Pilot mode allows mailing only to certain abuse groups, while redirecting others to system admins
    if ($self->{PILOT_MODE}) {
        if (ref $self->{PILOT_MODE} eq 'HASH' and $self->{PILOT_MODE}->{$report->{abuse}}) {
            Log::Loger->warn(__PACKAGE__, "[EML] Running in pilot mode, redirecting report for abuse contact '$report->{abuse}' from <$report->{to}> to <".$self->{PILOT_MODE}->{$report->{abuse}}.">");
            $report->{to} = $self->{PILOT_MODE}->{$report->{abuse}};
        }
        else {
            Log::Loger->warn(__PACKAGE__, "[EML] Running in pilot mode, redirecting report for abuse contact '$report->{abuse}' from <$report->{to}> to system developers at <$self->{DEVEL_C}>");
            $report->{message} = "WARNING: Running in pilot mode, report for abuse contact '$report->{abuse}' was redirected from <$report->{to}> to system developers at <$self->{DEVEL_C}>.\n\n$report->{message}";
            $report->{to} = $self->{DEVEL_C};
        }
    }
    # Test data mode, redirect to test data recipients
    if ($self->{TEST_DATA}) {
        Log::Loger->warn(__PACKAGE__, "[EML] Running in test data mode, redirecting report for abuse contact '$report->{abuse}' from <$report->{to}> to test data recipients at <$self->{TEST_C}>");
        $report->{message} = "WARNING: Running in test data mode, report for abuse contact '$report->{abuse}' was redirected from <$report->{to}> to test data recipients at <$self->{TEST_C}>.\n\n$report->{message}";
        $report->{to} = $self->{TEST_C};
    }
    # Test mode beats everything, redirect to system developers
    if ($self->{TEST_MODE}) {
        Log::Loger->warn(__PACKAGE__, "[EML] Running in test mode, redirecting report for abuse contact '$report->{abuse}' from <$report->{to}> to system developers at <$self->{DEVEL_C}>");
        $report->{message} = "WARNING: Running in test mode, report for abuse contact '$report->{abuse}' was redirected from <$report->{to}> to system developers at <$self->{DEVEL_C}>.\n\n$report->{message}";
        $report->{to} = $self->{DEVEL_C};
    }
}

# _setup_headers($$;%) [PROTECTED]
#
#  Usage     : $self->_setup_headers($report, $email, %config)
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing

sub _setup_headers($$;%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report, $email, %config) = @_;

    # Prepare the email skeleton with standard headers
    $email->{'From'}     = $self->{FROM};
    $email->{'To'}       = $report->{to};
    $email->{'Reply-To'} = $self->{REPLY_TO} if $self->{REPLY_TO};
    $email->{'Bcc'}      = $self->{BCC_C}    if $self->{BCC_C};

    # Insert optional custom email extended headers
    $email->{'X-Cesnet-Report-Id'}       = $report->{id};
    $email->{'X-Cesnet-Report-Type'}     = $report->{type};
    $email->{'X-Cesnet-Report-Severity'} = $report->{severity} if $report->{severity};
    $email->{'X-Cesnet-Report-Srcip'}    = $report->{ip}       if $report->{ip};
    $email->{'X-Cesnet-Report-Testdata'} = "Yes"               if $self->{TEST_DATA};
    $email->{'X-Cesnet-Report-Testmode'} = "Yes"               if $self->{TEST_MODE};
}

# _setup_subject($$;%) [PROTECTED]
#
#  Usage     : $self->_setup_subject($report, $email, %config)
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing

sub _setup_subject($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report, $email, %config) = @_;

    my %subjects = (
        'low'      => 'Upozorneni na mozne problemy ve Vasi siti',
        'medium'   => 'Upozorneni na mozne problemy ve Vasi siti',
        'high'     => 'Upozorneni na mozne vazne problemy ve Vasi siti',
        'critical' => 'Upozorneni na mozne kriticke problemy ve Vasi siti',
    );
    my $label = "Upozorneni na mozne problemy ve Vasi siti";
    if ($report->{severity} and exists $subjects{$report->{severity}}) {
        $label = ucfirst($report->{severity}) . ' - ' . $subjects{$report->{severity}};
    }
    $email->{Subject} = (($self->{PILOT_MODE} || $self->{TEST_DATA}) ? "TEST " : '') . "[$report->{id}] $label" . (($report->{type} and $report->{type} eq TYPE_SUMMARY()) ? ' (souhrn)' : ' ('.$report->{ip}.')');

    # This feature was removed on the user request, some ticket systems were unable to handle subject encoding
    #{
    #   use utf8;
    #
    #   my %subjects = (
    #       'low'      => 'Upozornění na drobné problémy ve Vaší síti',
    #       'medium'   => 'Hlášení vážných problémů ve Vaší síti',
    #       'high'     => 'Hlášení velmi vážných problémů ve Vaší síti',
    #       'critical' => 'Hlášení kritických problémů ve Vaší síti',
    #   );
    #   my $label = "Hlášení problémů ve Vaší síti";
    #   if ($report->{severity} and exists $subjects{$report->{severity}}) {
    #       $label = ucfirst($report->{severity}) . ' - ' . $subjects{$report->{severity}};
    #   }
    #   $email->{Subject} = encode("MIME-Header", (($self->{PILOT_MODE}) ? "TEST " : '') . "[$report->{id}] $label" . (($report->{type} and $report->{type} eq TYPE_SUMMARY())?' (souhrn)':''));
    #}
}

# _setup_message($$;%) [PROTECTED]
#
#  Usage     : $self->_setup_message($report, $email, %config)
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing

sub _setup_message($$;%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report, $email, %config) = @_;

    # Normal processing, when attachment file exists
    if ($report->{attachment_file1} or $report->{attachment_file2}) {
        my $boundary = _email_create_boundary();

        $email->{'Content-Type'} = qq(multipart/mixed; boundary="$boundary");
        # Attach the message body --------------------------------------------------
        $email->{'Message'} = "This is a multi-part message in MIME format.\n"
        . "--$boundary\n"
        . "Content-Type: text/plain; charset=utf-8\n"
        . "Content-Transfer-Encoding: 8bit\n"
        . "Content-Disposition: inline\n\n"
        #. "Target abuse contact: $report->{abuse}\n\n"
        . "$report->{message}\n"
        . "--$boundary\n";
        if ($report->{attachment_file1} and -f "".$self->{REPORTS_DIR}."/".$report->{attachment_file1}."") {
            $email->{'Message'} .= _email_attach_file($self->{REPORTS_DIR}, $report->{attachment_file1}, $boundary);
        }
        elsif ($report->{attachment_file1}) {
            Log::Loger->debug(__PACKAGE__, "[EML] Report *$report->{id}*: First data attachment file ".$self->{REPORTS_DIR}."/".$report->{attachment_file1}." does not exist");
        }
        else {
            Log::Loger->debug(__PACKAGE__, "[EML] Report *$report->{id}*: First data attachment file was not included");
        }
        if ($report->{attachment_file2} and -f "".$self->{REPORTS_DIR}."/".$report->{attachment_file2}."") {
            $email->{'Message'} .= _email_attach_file($self->{REPORTS_DIR}, $report->{attachment_file2}, $boundary);
        }
        elsif ($report->{attachment_file2}) {
            Log::Loger->debug(__PACKAGE__, "[EML] Report *$report->{id}*: Second data attachment file ".$self->{REPORTS_DIR}."/".$report->{attachment_file2}." does not exist");
        }
        else {
            Log::Loger->debug(__PACKAGE__, "[EML] Report *$report->{id}*: Second data attachment file was not included");
        }
    }
    elsif ($report->{cfg_attach} and $report->{cfg_attach} eq 'none') {
        $email->{'Content-Type'} = 'text/plain; charset="utf-8"';
        $email->{'Message'} = "$report->{message}\n";
    }
    # Fallback in case of attachment file is missing
    else {
        Log::Loger->error(__PACKAGE__, "[EML] Report *$report->{id}*: Data attachment file $self->{REPORTS_DIR}/$report->{attachment_file1} does not exist");
        $email->{'Content-Type'} = 'text/plain; charset="utf-8"';
        $email->{'Message'} = "$report->{message}\n\nThere are no additional data files attached to this message !!!\n";
    }
}

# _send_report_email($$;%) [PROTECTED]
#
#  Usage     : $self->_send_report_email($report, $email, %config)
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Nothing

sub _send_report_email($$;%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($report, $email, %config) = @_;

    my $success = sendmail(%$email);
    if ($success) {
        my $result;
        if ($Mail::Sendmail::log =~ /(Result: [^\n]+)/) { $result = $1; }
        Log::Loger->notice(__PACKAGE__, "[EML] Report *$report->{id}*: Successfully mailed report for abuse contact <$report->{abuse}> to <$report->{to}>: " . $result);
        #Log::Loger->notice(__PACKAGE__, "[EML] Report *$report->{id}*: Successfully mailed report for abuse contact <$report->{abuse}> to <$report->{to}>: " . $Mail::Sendmail::log);
        $result = ($report->{abuse} eq $report->{to}) ? "Report mailed to abuse contact '$report->{to}'" : "Report for abuse contact '$report->{abuse}' mailed to '$report->{to}'";

        return ($success, $result, $report->{to});
    }
    else {
        if ($Mail::Sendmail::error =~ /message file too big/) {
            Log::Loger->error(__PACKAGE__, "[EML] Report *$report->{id}*: Unable to mail report for abuse contact <$report->{abuse}> to <$report->{to}>: " . $Mail::Sendmail::error);

            return ($success, $Mail::Sendmail::error);
        }
        else {
            Log::Loger->error(__PACKAGE__, "[EML] Report *$report->{id}*: Unable to mail report for abuse contact <$report->{abuse}> to <$report->{to}>: " . $Mail::Sendmail::error);

            return ($success, $Mail::Sendmail::error);
        }
    }
}

#-------------------------------------------------------------------------------

# _email_create_content_id() [PROTECTED]
#
#  Usage     : my $id = _email_create_content_id();
#  Purpose   : Create unique content ID
#  Arguments : None
#  Returns   : STRING $id
#  Throws    : Nothing

sub _email_create_content_id()
{
    my $unique_string = rand(100);
    $unique_string = $unique_string . substr(md5_hex(time()),0,23);
    $unique_string =~ s/(.{5})/$1\./g;
    $unique_string = qq(part.${unique_string}\@) . "MAIL";
    return $unique_string;
}

# _email_create_boundary() [PROTECTED]
#
#  Usage     : my $boundary = _email_create_boundary();
#  Purpose   : Create boundary string for multipart emails
#  Arguments : None
#  Returns   : STRING $boundary
#  Throws    : Nothing

sub _email_create_boundary()
{
    my $unique_string  = substr(md5_hex(time()),0,24);
    $unique_string     = '======' . $unique_string . '======';
    return $unique_string;
}

# _email_encode_attachment($$) [PROTECTED]
#
#  Usage     : my $encoded = _email_encode_attachment($dir, $file);
#  Purpose   : Base64 encode given file in target directory
#  Arguments : STRING $dir - directory containing the report files [MANDATORY]
#              STRING $name - name of the file within the directory above to encode [MANDATORY]
#  Returns   : STRING $encoded Base64 encoded file
#  Throws    : Nothing

sub _email_encode_attachment($$)
{
    my ($dir, $name) = @_;

    my $filename = "$dir/$name";
    if ($filename =~ /(.*)/) { $filename = $1; }

    Log::Loger->debug(__PACKAGE__, "[EML] Encoding file '$name' into Base64 encoded email attachment");

    # Base64 encode given file
    my $result = '';
    my $buf;
    open(FILE, "$filename") or die "Can't open attachment file '$name': $!";
    while (read(FILE, $buf, 60*57)) {
        $result .= encode_base64($buf);
    }
    close FILE;

    Log::Loger->debug(__PACKAGE__, "[EML] Encoded file '$name' into Base64 encoded email attachment");

    # Return result
    return $result;
}

# _email_attach_file($$$) [PROTECTED]
#
#  Usage     : my $attachment = _email_attach_file($dir, $attachment, $boundary);
#  Purpose   : Create boundary string for multipart emails
#  Arguments : None
#  Returns   : STRING $boundary
#  Throws    : Nothing

sub _email_attach_file($$$)
{
    my ($dir, $attachment, $boundary) = @_;

    my $attach_type = "text/plain";
    my $attach_name = $attachment;
    #if ($attachment =~ /json$/) {
    #    $attach_name =~ s/json$/txt/;
    #}
    $attach_type = "application/json" if $attachment =~ /json$/;
    $attach_type = "application/zip"  if $attachment =~ /zip$/;

    # Attach the data file -------------------------------------------------
    return "Content-Type: $attach_type; name=\"$attach_name\"\n"
         . "Content-Transfer-Encoding: base64\n"
         . "Content-Disposition: attachment; filename=\"$attach_name\"\n"
         . "Content-ID: <"._email_create_content_id().">\n"
         . "Content-Description: Related security data\n\n"
         . _email_encode_attachment($dir, "$attachment")."\n"
         . "--$boundary\n";
}

=pod

=back

=cut

1;
