package Mentat::Message::Formater::Helper;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Message::Formater::Helper - Short module description

=head1 SYNOPSIS

 use Mentat::Message::Formater::Helper;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Parent::Module(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Text::Wrap qw(fill $columns);

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#
use Number::Format qw(:subs :vars);

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:func);
use Config::JSON;
use Value::Convertor;
use DNS::Resolver;
use Mentat::Storage::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant M_UNKNOWN   => '_unknown';
use constant M_UNDEFINED => 'UNDEFINED';

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#
my $SERVICE_INSTANCE;
my $SINGLETON_INSTANCE;

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

# After problems with encoding ended up hardcoding everything here
my %DICT = (
        "report_extra"   => "Extra report",
        "report_summary" => "Souhrnný report",
        "extra"          => "Extra report",
        "summary"        => "Souhrnný report",
        "label_e"        => "Jiný typ útoku/problému/události",
        "label_s"        => "Jiné typy útoků/problémů/událostí"
    );
my %PLURALS = (
        "uaddress" => ["unikátní IP adresa","unikátní IP adresy","unikátních IP adres"],
        "attempts" => ["pokus","pokusy","pokusů"],
        "events"   => ["událost","události","událostí"]
    );

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item service() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Message::Formater::Helper->service()
  Purpose   : Create/return reference to the singleton instance
  Arguments : Same as the the new() method
  Returns   : Same as the the new() method
  Throws    : Same as the the new() method
  Comments  : Internally uses new() method
  See Also  : new() method

=cut

sub service()
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my ($config_file, $config_schema_file, $config_over) = @_;
    unless ($SERVICE_INSTANCE) {
        $config_file = loc_cre().'/svc.mentat-message-formater-helper.json' unless $config_file;
        my $cfg = $class->_load_config($config_file, $config_schema_file, $config_over);
        $SERVICE_INSTANCE = $class->new(%$cfg);
    }
    return $SERVICE_INSTANCE;
}

=item singleton() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Message::Formater::Helper->singleton()
  Purpose   : Create/return reference to the singleton instance
  Arguments : Same as the the new() method
  Returns   : Same as the the new() method
  Throws    : Same as the the new() method
  Comments  : Internally uses new() method
  See Also  : new() method

=cut

sub singleton
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    unless ($SINGLETON_INSTANCE) {
        $SINGLETON_INSTANCE = $class->new(@_);
    }
    return $SINGLETON_INSTANCE;
}

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Message::Formater::Helper->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Message::Formater::Helper reference on success, croaks on failure
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item dump(@) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub dump(@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return Dumper(@_);
}

=item conv($;@) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub conv($;@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($conversion, @args) = @_;

    return Value::Convertor->$conversion(@args);
}

=item time(;$$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub time($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($time, $print_in_local) = @_;

    $time = time unless $time;
    return Value::Convertor->unixs_to_datestr($time, $print_in_local);
}

=item unesc($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub unesc($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my $value = shift;
    $value =~ s/\(dot\)/./g;
    $value =~ s/\(at\)/@/g;
    return return $value;
}

=item time(;$$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub fmtn($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return format_number(shift);
}

=item plural($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub plural($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($name, $count) = @_;

    return $name unless %PLURALS and $PLURALS{$name};
    return $name."???" unless defined $count;

    return $PLURALS{$name}->[0] if $count == 1;
    return $PLURALS{$name}->[1] if $count > 1 and $count < 5;
    return $PLURALS{$name}->[2];
}

=item anonymize($;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub anonymize($;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ip, $level) = @_;

    $level = 1 unless $level;

    if ($level == 1) {
        $ip =~ s/\.[\d]+$/.???/;
        return $ip;
    }
    if ($level == 2) {
        $ip =~ s/\.[\d]+\.[\d]+$/.???.???/;
        return $ip;
    }
    $ip =~ s/\.[\d]+\.[\d]+\.[\d]+$/.???.???.???/;
    return $ip;
}

=item wrap($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub wrap($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($what,) = @_;

    $columns = 90;
    return fill("\t", "\t", $what);
}

=item meta($$$;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub meta($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($id, $type, $undefined) = @_;
    confess "Metadata ID expected to be given as argument" unless $id;
    Log::Loger->debug(__PACKAGE__, "[TH] Looking for metadadata for '$id':'$type'") if $DEVEL;

    if ($self->{META}
        and ref $self->{META} eq 'HASH'
        and exists $self->{META}->{$id}
        and ref $self->{META}->{$id} eq 'HASH'
        and exists $self->{META}->{$id}->{$type}) {

        use utf8;
        my $rv = $self->{META}->{$id}->{$type};

        if (utf8::is_utf8($rv)) {
            Log::Loger->debug(__PACKAGE__, "[TH] $type value is UTF8") if $DEVEL;
            use Encode;
            $rv = encode_utf8($rv);
        }
        else {
            Log::Loger->debug(__PACKAGE__, "[TH] $type value not UTF8") if $DEVEL;
        }
        Log::Loger->debug(__PACKAGE__, "[TH] Metadadata hit for '$id':'$type' = '$rv'") if $DEVEL;
        return $rv;
    }

    # Last chance - look into dictionary
    return $self->dict($type, $undefined);
}

=item dict($;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub dict($;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($key, $undefined) = @_;
    confess "Key expected to be given as argument" unless $key;
    Log::Loger->debug(__PACKAGE__, "[TH] Looking for dictdata for $key") if $DEVEL;

    if (%DICT and exists $DICT{$key}) {
        return $DICT{$key};
    }

    Log::Loger->error(__PACKAGE__, "[TH] Undefined dictdata for $key");
    return undef if $undefined;
    return M_UNDEFINED();
}

=item resolve(;$$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub resolve(;$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ip, $hostname) = @_;

    if ($self->{RESOLVER}) {
        #Log::Loger->info(__PACKAGE__, "[TH] Resolving '$ip'");
        ($ip, $hostname) = $self->{RESOLVER}->resolve($ip, $hostname);
    }
    return ($ip, $hostname);
}

=item fetch_values($@) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub fetch_values($@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message, @mpaths) = @_;

    my @result = ();
    my $tmp;
    foreach my $mp (@mpaths) {
        $tmp = $message->path_value($mp);
    }
    return @result;
}

=item events_aggregate($$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub events_aggregate($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my @messages = @_;
    #Log::Loger->info(__PACKAGE__, "[TH] Beginning aggregation of ".(scalar @messages)." messages");

    my $res = {};
    my ($ct, $dt);
    foreach my $m (@messages) {
        # Retrieve the alert timestamps
        $ct = $m->path_value('CreateTime');
        $dt = $m->path_value('DetectTime');

        # Increment the event counter
        $res->{'count'}++;

        # Calculate the min and max CreateTime timestamps
        if (defined $ct) {
            unless ($res->{'start'} and $res->{'end'}) {
                $res->{'start'} = $res->{'end'} = $ct;
            }
            unless ($res->{'ct_start'} and $res->{'ct_end'}) {
                $res->{'ct_start'} = $res->{'ct_end'} = $ct;
            }
            $res->{'ct_start'} = $ct if ($res->{'ct_start'} > $ct);
            $res->{'ct_end'}   = $ct if ($res->{'ct_end'}   < $ct);
            $res->{'start'}    = $ct if ($res->{'start'}    > $ct);
            $res->{'end'}      = $ct if ($res->{'end'}      < $ct);
        }
        # Calculate the min and max DetectTime timestamps
        if (defined $dt) {
            unless ($res->{'start'} and $res->{'end'}) {
                $res->{'start'} = $res->{'end'} = $dt;
            }
            unless ($res->{'dt_start'} and $res->{'dt_end'}) {
                $res->{'dt_start'} = $res->{'dt_end'} = $dt;
            }
            $res->{'dt_start'} = $dt if ($res->{'dt_start'} > $dt);
            $res->{'dt_end'}   = $dt if ($res->{'dt_end'}   < $dt);
            $res->{'start'}    = $dt if ($res->{'start'}    > $dt);
            $res->{'end'}      = $dt if ($res->{'end'}      < $dt);
        }
    }

    #Log::Loger->info(__PACKAGE__, "[TH] Done with message aggregation");
    return $res;
}

=item stats() [PUBLIC]

  Usage     : my $stats = $helper->stats();
  Purpose   : Get helper statistics
  Arguments : NONE
  Returns   : HASH REFERENCE $stats
  Throws    : Croaks, if invoked on object

=cut

sub stats()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return $self->{RESOLVER}->stats() if $self->{RESOLVER};
    return {};
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Message::Formater::Helper instance
#  Arguments :
#  Returns   : Mentat::Message::Formater::Helper reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %cfg = @_;

    $self->{RESOLVER} = DNS::Resolver->new() if $cfg{resolve};
    $self->{META}     = $self->_load_templates_db($cfg{event_database}, $cfg{event_collection}) if $cfg{event_database} and $cfg{event_collection};
    #$self->{META}    = $self->_load_templates($cfg{template_file}, $cfg{template_schema_file}) if $cfg{template_file};

    return $self;
}

# _load_config($;$) [PROTECTED]
#
#  Usage     : my $config = $self->_load_config($config_file, $config_schema_file);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on instance

sub _load_config($;$)
{
    my $invocant = shift;
    my ($config_file, $config_schema_file, $config_over) = @_;

    my $cfg_parser = Config::JSON->new($config_schema_file);

    # Load the templates from external file
    my ($config, $error);
    #($config, $error) = $cfg_parser->config($config_file, $config_schema_file);
    #die 'Template file syntax error: ' . $error . "\n" unless $config;

    $config = json_load($config_file);

    if ($config_over and ref $config_over eq 'HASH') {
        foreach my $c (keys %$config_over) {
            $config->{$c} = $config_over->{$c};
        }
    }

    return $config;
}

# _load_templates($$) [PROTECTED]
#
#  Usage     : $self->{TEMPLATES} = $self->_load_templates($template_file, $template_schema_file);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on instance

sub _load_templates($$)
{
    my $invocant = shift;
    my ($template_file, $template_schema_file) = @_;

    my $cfg_parser = Config::JSON->new($template_schema_file);

    # Load the templates from external file
    my ($config, $error);
    ($config, $error) = $cfg_parser->config($template_file, $template_schema_file);
    die 'Template file syntax error: ' . $error . "\n" unless $config;

    $config = $config->{templates};

    my $meta = {};
    foreach my $key (keys %$config) {
        $meta->{$config->{$key}->{id}} = $config->{$key};
    }
    return $meta;
}

# _load_templates_db($$) [PROTECTED]
#
#  Usage     : $self->{TEMPLATES} = $self->_load_templates_db($event_database, $event_collection);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on instance

sub _load_templates_db($$)
{
    my $invocant = shift;
    my ($event_database, $event_collection) = @_;

    Log::Loger->debug(__PACKAGE__, "[TH] Loading events from $event_database:$event_collection") if $DEVEL;

    my $db = Mentat::Storage::Mongo->new(database => $event_database, collection => $event_collection);
    my ($events, $count_all) = $db->find({'enabled' => 1});
    $events = [] unless $count_all;

    my $meta = {};
    foreach my $e (@$events) {
        $meta->{$e->{id}} = $e if $e->{id};
    }
    Log::Loger->debug(__PACKAGE__, "[TH] Loaded events from $event_database:$event_collection: ".Dumper($meta)) if $DEVEL;
    return $meta;
}

=pod

=back

=cut

1;
